@@ -7,7 +7,7 @@ from mojo.apps.account.services.push import (
     send_push_notification, send_direct_notification
 )
 from mojo.helpers import response
-
+from mojo.helpers import request as rhelper
 
 @md.POST('account/devices/push/register')
 @md.requires_auth()
@@ -53,17 +53,22 @@ def register_device(request):
 
 @md.POST('account/member/device/register')
 def register_legacy_device(request):
-    device_id = request.DATA.get("device_id")
-    cmf_token = request.DATA.get("cmf_token")
+    device_id = request.DATA.get("device_id", request.duid)
+    if not device_id:
+        device_id = request.device.id
+    cmf_token = request.DATA.get(["cmf_token", "cm_token"])
+    meta = request.DATA.get("device_metadata", {})
+    ua_info = rhelper.parse_user_agent(request.user_agent)
+    platform = ua_info.os.family
     device, created = RegisteredDevice.objects.update_or_create(
         user=request.user,
         device_id=device_id,
         defaults={
             'device_token': cmf_token,
-            'platform': request.DATA.get('platform', ''),
-            'device_name': request.DATA.get('device_name', ''),
-            'app_version': request.DATA.get('app_version', ''),
-            'os_version': request.DATA.get('os_version', ''),
+            'platform': platform.lower(),
+            'device_name': f"{request.user.display_name} {{platform}}",
+            'app_version': meta.get('app_version', ''),
+            'os_version': meta.get('os_version', ''),
             'push_preferences': request.DATA.get('push_preferences', {}),
             'is_active': True,
             'push_enabled': True
