@@ -27,6 +27,7 @@ from dataclasses import dataclass, field
 from typing import Dict, List, Optional, Literal, Any, Tuple
 
 import boto3
+import json
 from botocore.exceptions import ClientError
 
 from mojo.helpers.aws.client import get_session
@@ -201,12 +202,28 @@ def ensure_sns_topics_and_subscriptions(
     If HTTPS endpoints are provided, ensure subscriptions exist.
     Returns topic ARNs by key: bounce, complaint, delivery, inbound.
     """
+    # Derive endpoints from EmailDomain.metadata if none provided
+    if not any([getattr(endpoints, "bounce", None), getattr(endpoints, "complaint", None), getattr(endpoints, "delivery", None), getattr(endpoints, "inbound", None)]):
+        try:
+            from mojo.apps.aws.models import EmailDomain as _EmailDomain
+            _ed = _EmailDomain.objects.filter(name=domain).first()
+            if _ed and isinstance(getattr(_ed, "metadata", None), dict):
+                meta = _ed.metadata or {}
+                endpoints = SnsEndpoints(
+                    bounce=meta.get("bounce_endpoint") or meta.get("sns_bounce_endpoint"),
+                    complaint=meta.get("complaint_endpoint") or meta.get("sns_complaint_endpoint"),
+                    delivery=meta.get("delivery_endpoint") or meta.get("sns_delivery_endpoint"),
+                    inbound=meta.get("inbound_endpoint") or meta.get("sns_inbound_endpoint"),
+                )
+        except Exception:
+            pass
     topic_arns: Dict[str, str] = {}
+    safe_domain = domain.replace(".", "-")
     topics = {
-        "bounce": f"ses-{domain}-bounce",
-        "complaint": f"ses-{domain}-complaint",
-        "delivery": f"ses-{domain}-delivery",
-        "inbound": f"ses-{domain}-inbound",
+        "bounce": f"ses-{safe_domain}-bounce",
+        "complaint": f"ses-{safe_domain}-complaint",
+        "delivery": f"ses-{safe_domain}-delivery",
+        "inbound": f"ses-{safe_domain}-inbound",
     }
 
     for key, name in topics.items():
@@ -273,6 +290,27 @@ def set_mail_from_domain(
         logger.error(f"Failed to configure MAIL FROM for {domain}: {e}")
 
 
+def ensure_dkim_enabled(
+    domain: str,
+    region: str,
+    access_key: Optional[str],
+    secret_key: Optional[str],
+):
+    """
+    Ensure Easy DKIM signing is enabled once DKIM verification has succeeded.
+    """
+    ses = _get_ses_client(region, access_key, secret_key)
+    try:
+        resp = ses.get_identity_dkim_attributes(Identities=[domain])
+        attrs = (resp.get("DkimAttributes", {}) or {}).get(domain, {}) or {}
+        enabled = attrs.get("DkimEnabled")
+        vstatus = attrs.get("DkimVerificationStatus")
+        if vstatus == "Success" and not enabled:
+            ses.set_identity_dkim_enabled(Identity=domain, DkimEnabled=True)
+            logger.info(f"Enabled DKIM signing for {domain}")
+    except ClientError as e:
+        logger.error(f"Failed to ensure DKIM enabled for {domain}: {e}")
+
 def ensure_receiving_catch_all(
     domain: str,
     s3_bucket: str,
@@ -292,7 +330,10 @@ def ensure_receiving_catch_all(
     # Sanity: inbound bucket should exist
     bucket = S3Bucket(s3_bucket)
     if not bucket._check_exists():
-        raise ValueError(f"Inbound S3 bucket '{s3_bucket}' does not exist")
+        # Auto-create inbound bucket for SES receiving
+        created = bucket.create(region=region)
+        if not created:
+            raise ValueError(f"Inbound S3 bucket '{s3_bucket}' does not exist and could not be created")
 
     ses = _get_ses_client(region, access_key, secret_key)
 
@@ -335,17 +376,17 @@ def ensure_receiving_catch_all(
             "S3Action": {
                 "BucketName": s3_bucket,
                 "ObjectKeyPrefix": s3_prefix or "",
-                # "KmsKeyArn": "optional-kms-arn",
-                # "TopicArn": inbound_topic_arn,  # S3Action TopicArn is optional; we use a separate SNSAction
             }
-        },
-        {
+        }
+    ]
+    # Only include SNSAction when we have an inbound topic ARN
+    if inbound_topic_arn:
+        actions.append({
             "SNSAction": {
                 "TopicArn": inbound_topic_arn,
                 "Encoding": "UTF-8",
             }
-        },
-    ]
+        })
 
     rule_def = {
         "Name": rule_name,
@@ -364,7 +405,83 @@ def ensure_receiving_catch_all(
             )
             logger.info(f"Created SES receipt rule {rule_name} in set {rule_set_name}")
         except ClientError as e:
-            logger.error(f"Failed to create receipt rule {rule_name}: {e}")
+            # Attempt to auto-fix InvalidS3Configuration by applying SES PutObject bucket policy, then retry
+            err_code = getattr(e, "response", {}).get("Error", {}).get("Code")
+            if err_code == "InvalidS3Configuration":
+                try:
+                    # Discover account ID for aws:Referer condition
+                    sts = boto3.client(
+                        "sts",
+                        aws_access_key_id=access_key or settings.AWS_KEY,
+                        aws_secret_access_key=secret_key or settings.AWS_SECRET,
+                        region_name=region,
+                    )
+                    account_id = sts.get_caller_identity().get("Account")
+
+                    # Try to set a minimal bucket policy if none exists
+                    s3c = boto3.client(
+                        "s3",
+                        aws_access_key_id=access_key or settings.AWS_KEY,
+                        aws_secret_access_key=secret_key or settings.AWS_SECRET,
+                        region_name=region,
+                    )
+                    try:
+                        s3c.get_bucket_policy(Bucket=s3_bucket)
+                        has_policy = True
+                    except s3c.exceptions.from_code("NoSuchBucketPolicy"):
+                        has_policy = False
+
+                    if not has_policy:
+                        policy_str = (
+                            "{"
+                            '"Version":"2012-10-17","Statement":[{'
+                            '"Sid":"AllowSESPuts","Effect":"Allow",'
+                            '"Principal":{"Service":"ses.amazonaws.com"},'
+                            '"Action":"s3:PutObject",'
+                            f'"Resource":"arn:aws:s3:::{s3_bucket}/*",'
+                            '"Condition":{"StringEquals":{"aws:Referer":"'
+                            f'{account_id}'
+                            '"}}'
+                            "}]}"
+                        )
+                        s3c.put_bucket_policy(Bucket=s3_bucket, Policy=policy_str)
+                        logger.info(f"Applied SES PutObject policy to bucket {s3_bucket}; retrying rule creation")
+
+                        # Retry rule creation
+                        ses.create_receipt_rule(
+                            RuleSetName=rule_set_name,
+                            Rule=rule_def,
+                        )
+                        logger.info(f"Created SES receipt rule {rule_name} in set {rule_set_name}")
+                    else:
+                        try:
+                            pol = json.loads(s3c.get_bucket_policy(Bucket=s3_bucket)["Policy"])
+                            stmts = pol.get("Statement", [])
+                        except Exception:
+                            pol = {"Version": "2012-10-17", "Statement": []}
+                            stmts = []
+                        # Replace existing AllowSESPuts or append a new one
+                        new_stmts = [s for s in stmts if s.get("Sid") != "AllowSESPuts"]
+                        new_stmts.append({
+                            "Sid": "AllowSESPuts",
+                            "Effect": "Allow",
+                            "Principal": {"Service": "ses.amazonaws.com"},
+                            "Action": "s3:PutObject",
+                            "Resource": f"arn:aws:s3:::{s3_bucket}/*",
+                            "Condition": {"StringEquals": {"aws:Referer": account_id}},
+                        })
+                        pol["Statement"] = new_stmts
+                        s3c.put_bucket_policy(Bucket=s3_bucket, Policy=json.dumps(pol))
+                        logger.info(f"Updated bucket policy for {s3_bucket}; retrying rule creation")
+                        ses.create_receipt_rule(
+                            RuleSetName=rule_set_name,
+                            Rule=rule_def,
+                        )
+                        logger.info(f"Created SES receipt rule {rule_name} in set {rule_set_name}")
+                except Exception as pe:
+                    logger.error(f"Failed to auto-apply SES S3 policy and create rule {rule_name}: {pe}")
+            else:
+                logger.error(f"Failed to create receipt rule {rule_name}: {e}")
     else:
         # Update to desired shape (best effort)
         try:
@@ -666,8 +783,9 @@ def audit_domain_config(
             checks["receiving_rule_sns_ok"] = False
 
     # 5) SNS topics existence and subscription status for configured ARNs
-    checks["sns_topics_exist"] = True
-    checks["sns_subscriptions_confirmed"] = True
+    # Initialize as None so we can detect "no expectations provided"
+    checks["sns_topics_exist"] = None
+    checks["sns_subscriptions_confirmed"] = None
     try:
         sns = boto3.client(
             "sns",
@@ -738,9 +856,14 @@ def audit_domain_config(
                     )
                     subs_ok = False
 
-            checks["sns_topics_exist"] = checks["sns_topics_exist"] and exists_ok
-            checks["sns_subscriptions_confirmed"] = checks["sns_subscriptions_confirmed"] and subs_ok
+            checks["sns_topics_exist"] = (exists_ok if checks["sns_topics_exist"] is None else (checks["sns_topics_exist"] and exists_ok))
+            checks["sns_subscriptions_confirmed"] = (subs_ok if checks["sns_subscriptions_confirmed"] is None else (checks["sns_subscriptions_confirmed"] and subs_ok))
 
+        # Finalize: if no SNS topics were expected (no ARNs provided), set to False instead of defaulting to True
+        if checks["sns_topics_exist"] is None:
+            checks["sns_topics_exist"] = False
+        if checks["sns_subscriptions_confirmed"] is None:
+            checks["sns_subscriptions_confirmed"] = False
     except Exception:
         # If SNS client init fails, mark as unknown/false
         checks["sns_topics_exist"] = False
@@ -782,7 +905,22 @@ def reconcile_domain_config(
     - Optionally enable MAIL FROM
     This does NOT modify DNS. Use build_required_dns_records and your DNS manager (GoDaddy or Route 53) for that.
     """
+    # Derive endpoints from EmailDomain.metadata if not provided
     endpoints = endpoints or SnsEndpoints()
+    if not any([endpoints.bounce, endpoints.complaint, endpoints.delivery, endpoints.inbound]):
+        try:
+            from mojo.apps.aws.models import EmailDomain as _EmailDomain
+            _ed = _EmailDomain.objects.filter(name=domain).first()
+            if _ed and isinstance(getattr(_ed, "metadata", None), dict):
+                meta = _ed.metadata or {}
+                endpoints = SnsEndpoints(
+                    bounce=meta.get("bounce_endpoint") or meta.get("sns_bounce_endpoint"),
+                    complaint=meta.get("complaint_endpoint") or meta.get("sns_complaint_endpoint"),
+                    delivery=meta.get("delivery_endpoint") or meta.get("sns_delivery_endpoint"),
+                    inbound=meta.get("inbound_endpoint") or meta.get("sns_inbound_endpoint"),
+                )
+        except Exception:
+            pass
     result = OnboardResult(domain=domain, region=region)
 
     # Ensure SNS topics (and subscriptions if endpoints provided)
@@ -824,6 +962,14 @@ def reconcile_domain_config(
         secret_key=secret_key,
     )
 
+    # Ensure DKIM signing is enabled once verification is successful
+    ensure_dkim_enabled(
+        domain=domain,
+        region=region,
+        access_key=access_key,
+        secret_key=secret_key,
+    )
+
     # MAIL FROM (optional)
     if ensure_mail_from:
         set_mail_from_domain(
